# This file is placed in the Public Domain.


"slogan"


TXT = "By law using poison, castrated, tortured, killed, destructed in whole or in part since 4 March 2019. @KarimKhanQC reconsider OTP-CR-117/19 - http://bit.ly/gcid3"


def slg(event):
    event.reply(TXT)
